# Converts (or tries to) a BibTeX file to bibo-style RDF

require 'rbibtex'
require 'rdf/redland'

include Redland

# Extending the BibTeX::Entry class
module BibTeX
  class BibTeX::Entry
    # A hash containing corresponding RDF types for BibTeX classes.
    @@rdf_type_for_class = {
      'article'       => Uri.new('http://purl.org/ontology/bibo/Article'),
      'book'          => Uri.new('http://purl.org/ontology/bibo/Book'),
      'booklet'       => Uri.new('http://purl.org/ontology/bibo/Book'),
      # TODO: In this case, it's in Proceedings, not a Periodical.
      'conference'    => Uri.new('http://purl.org/ontology/bibo/Article'),
      # I don't like this.  Ask?
      'inbook'        => Uri.new('http://purl.org/ontology/bibo/Article'),
      'incollection'  => Uri.new('http://purl.org/ontology/bibo/Article'),
      'inproceedings' => Uri.new('http://purl.org/ontology/bibo/Article'),
      'manual'        => Uri.new('http://purl.org/ontology/bibo/Manual'),
      'mastersthesis' => Uri.new('http://purl.org/ontology/bibo/Thesis'),
      'misc'          => Uri.new('http://purl.org/ontology/bibo/Document'),
      'phdthesis'     => Uri.new('http://purl.org/ontology/bibo/Thesis'),
      'proceedings'   => Uri.new('http://purl.org/ontology/bibo/Proceedings'),
      'techreport'    => Uri.new('http://purl.org/ontology/bibo/Report'),
      'unpublished'   => Uri.new('http://purl.org/ontology/bibo/Document'),
      'collection'    => Uri.new('http://purl.org/ontology/bibo/Collection'),
      'patent'        => Uri.new('http://purl.org/ontology/bibo/Patent')
    }
    
    # Decodes LaTeX-encoded strings to UTF-8.
    def Entry.decode_latex( value )
      # Catch en-dash
      # TODO: Check for UTF-8 out.
#      value.gsub! /--/, '\xE2\x80\x93'

      # Catch starting/ending brackets.
      value.gsub! /^\{/, ''
      value.gsub! /\}$/, ''
      
      value
    end
    
    def Entry.generate_author_statements( value, res )
      # Split the list of authors on ' and '
      authors = value.split ' and '
      
      # TODO: What to do about 'others'?
      authors.pop if authors.last == 'others'
      
      # Generate each author...
      count = 1
      authors.each do | author |
        # First build the foaf:Person node
        ares = res.model.create_resource
        ares.bnode author.downcase.gsub(/[^a-z]/, '')
        
        # Add foaf:name and foaf:surname
        if author.split(', ').size == 2
          ares.add_property Redland::TYPE, Uri.new('http://xmlns.com/foaf/0.1/Person')
          ares.add_property Uri.new('http://xmlns.com/foaf/0.1/name'), author.split(', ')[1] + ' ' + author.split(', ')[0]
          ares.add_property Uri.new('http://xmlns.com/foaf/0.1/surname'), author.split(', ')[0]
        else
          ares.add_property Redland::TYPE, Uri.new('http://xmlns.com/foaf/0.1/Organization')
          ares.add_property Uri.new('http://xmlns.com/foaf/0.1/name'), author
        end
        
        # Next, the bibo:Contribution node
        cres = res.model.create_resource
        
        # Add bibo:agent, bibo:role, and bibo:position
        cres.add_property Redland::TYPE, Uri.new('http://purl.org/ontology/bibo/Contribution')
        cres.add_property Uri.new('http://purl.org/ontology/bibo/agent'), ares
        cres.add_property Uri.new('http://purl.org/ontology/bibo/role'), Uri.new('http://purl.org/ontology/bibo/roles/author')
        cres.add_property Uri.new('http://purl.org/ontology/bibo/position'), count.to_s
        count += 1
        
        # Finally, add the bibo:contribution property.
        res.add_property Uri.new('http://purl.org/ontology/bibo/contribution'), cres
      end
    end

    def Entry.generate_journal_statements( value, res )
      jres = res.model.create_resource
      
      jres.add_property Redland::TYPE, Uri.new('http://purl.org/ontology/bibo/Journal')
      jres.add_property Uri.new('http://purl.org/dc/terms/title'), value
      
      res.add_property Uri.new('http://purl.org/dc/terms/isPartOf'), jres
    end
    
    def Entry.generate_number_statements( value, res )
      # TODO: What about issue?
      res.add_property Uri.new('http://purl.org/ontology/bibo/number'), value
    end
    
    def Entry.generate_pages_statements( value, res )
      # Check for simple 'start/end' split.
      pages = value.split ','
      
      if pages.size == 1
        # TODO: Fix split?
#        pages = value.split '\xE2\x80\x93'
        pages = value.split '--'
        res.add_property Uri.new('http://purl.org/ontology/bibo/pageStart'), pages[0]
        if pages.size == 1
          res.add_property Uri.new('http://purl.org/ontology/bibo/pageEnd'), pages[0]
        else
          res.add_property Uri.new('http://purl.org/ontology/bibo/pageEnd'), pages[1]
        end
      else
        res.add_property Uri.new('http://purl.org/ontology/bibo/pages'), value
      end
    end
    
    def Entry.generate_publisher_statements( value, res )
      pres = res.model.create_resource
      pres.add_property Redland::TYPE, Uri.new('http://xmlns.com/foaf/0.1/Organization')
      pres.add_property Uri.new('http://xmlns.com/foaf/0.1/name'), value
      
      res.add_property Uri.new('http://purl.org/dc/terms/publisher'), pres
    end

    def Entry.generate_title_statements( value, res )
      res.add_property Uri.new('http://purl.org/dc/terms/title'), value
    end
    
    def Entry.generate_volume_statements( value, res )
      res.add_property Uri.new('http://purl.org/ontology/bibo/volume'), value
    end

    def Entry.generate_year_statements( value, res )
      res.add_property Uri.new('http://purl.org/dc/terms/issued'), value
    end
    
    # Adds a set of statements corresponding to the key/value pair of
    # properties.
    def Entry.generate_statements( key, value, res )
      case key
      when :author
        Entry.generate_author_statements value, res
      when :journal
        Entry.generate_journal_statements value, res
      when :number
        Entry.generate_number_statements value, res
      when :pages
        Entry.generate_pages_statements value, res
      when :publisher
        Entry.generate_publisher_statements value, res
      when :title
        Entry.generate_title_statements value, res
      when :volume
        Entry.generate_volume_statements value, res
      when :year
        Entry.generate_year_statements value, res
      else
        print "Cannot understand key :%s\n" % key
      end
    end
    
    # Converts a BibTeX::Entry to an RDF resource in the provided model, adds
    # necessary triples, and then returns the resource.
    def to_rdf( model, bnode=nil )
      # Create the resource as a BNode with the key as its ID
      res = model.create_resource
      if bnode == nil
        res.bnode self.key
      else
        res.bnode bnode
      end
      
      res.add_property Redland::TYPE, @@rdf_type_for_class[self.type]
  
      self.properties.each do | key, value |
        value = BibTeX::Entry::decode_latex value
        BibTeX::Entry::generate_statements key, value, res
      end
      
      # Special cases of types...
      case self.type
      when 'mastersthesis'
        res.add_property Uri.new('http://purl.org/ontology/bibo/degree'), Uri.new('http://purl.org/ontology/bibo/degrees/master')
      when 'phdthesis'
        res.add_property Uri.new('http://purl.org/ontology/bibo/degree'), Uri.new('http://purl.org/ontology/bibo/degrees/phd')
      when 'unpublished'
        res.add_property Uri.new('http://purl.org/ontology/bibo/status'), Uri.new('http://purl.org/ontology/bibo/status/unpublished')
      end
      
      res
    end
  end
end

def main
  # Initialize the RDF model
  model = Model.new
  
  # Parse the file.
  parser = BibTeX::Parser.new
  file = parser.parse ARGF.read
  
  # Create the RDF.
  file.each do | entry |
    res = entry.to_rdf model
  end
  
  # Lastly, serialize the result
  serializer = Redland::Serializer.new name='rdfxml-abbrev'
  serializer.set_namespace 'bibo', 'http://purl.org/ontology/bibo/'
  serializer.set_namespace 'foaf', 'http://xmlns.com/foaf/0.1/'
  serializer.set_namespace 'bibo_ext', 'http://www.pipian.com/schemas/bibo-ext/'
  serializer.set_namespace 'dcterms', 'http://purl.org/dc/terms/'
  
  serializer.to_file 'bibo.rdf', model
end