# Library that can parse a Google scholar page.
require 'bib2rdf'
require 'rdf/redland'

include Redland

class GoogleScholar
  def GoogleScholar.parse( html, cookie )
    # Results end after </font> before <div class=n>
    pos = html.rindex '<div class=n>'
    pos = html.rindex '</font>', pos
    pos += 7
    html.slice! pos, html.length
    
    # Each item starts with <p class=g>
    items = html.split '<p class=g>'

    # Ignore the first item, as it is the 'header' content to the results
    items.shift
    
    # Init the Redland model
    model = Model.new
    
    items.each do | item |
      resource = nil
      
      # Use "Cited by" link to get cluster ID.  Fall back on "all versions".
      # TODO: May be looking at individual document, not cluster
      cluster_id = nil
      pos = item.index 'Cited by'
      if pos == nil
        pos = item.index 'versions &raquo;'
        pos = item.rindex 'cluster=', pos if pos != nil
        cluster_id = item.slice pos + 8, item.index('"', pos) - pos - 8 if pos != nil
      else
        pos = item.rindex 'cites=', pos
        cluster_id = item.slice pos + 6, item.index('"', pos) - pos - 6 if pos != nil
      end
      
      # Use "Import into BibTeX" to get BibTeX info and document ID.
      document_id = nil
      pos = item.index 'Import into BibTeX'
      if pos != nil
        # TODO: May be looking at individual document, not cluster.
        pos = item.rindex 'href="', pos
        if pos != nil
          bibtexuri = item.slice pos + 6, item.index('"', pos + 6) - pos - 6
          
          # Get the Document ID
          pos = bibtexuri.index('info:')
          document_id = bibtexuri.slice pos + 5, bibtexuri.index(':', pos + 5) - pos - 5 if pos != nil

          # Get the BibTeX
          http = Net::HTTP.new 'scholar.google.com', 80
          http.start do | http |
            req = Net::HTTP::Get.new bibtexuri,
                {'Cookie', "GSP=#{cookie}"}
            response = http.request req
            bibtex = response.body
            parser = BibTeX::Parser.new
            file = parser.parse bibtex
            file.each do | entry |
              resource = entry.to_rdf model, document_id
            end
          end
        end
      end
      
      # Use "Find It @ Harvard" link to get ISSN/ISBN/DOI
      issn = nil
      isbn = nil
      doi = nil
      pos = item.index 'Find It @ Harvard'
      if pos != nil
        issnpos = item.rindex 'issn=', pos
        issn = item.slice issnpos + 5, item.index('"', issnpos) - issnpos - 5 if issnpos != nil
        isbnpos = item.rindex 'isbn=', pos
        isbn = item.slice isbnpos + 5, item.index('"', isbnpos) - isbnpos - 5 if isbnpos != nil
        doipos = item.rindex 'id=doi:', pos
        doi = item.slice doipos + 7, item.index('"', doipos) - doipos - 7 if doipos != nil
      end
      
      # Use "Library Search" link to get OCLC ID
      oclc = nil
      pos = item.index 'Library Search'
      if pos != nil
        pos = item.rindex 'oclc/', pos
        oclc = item.slice pos + 5, item.index('"', pos) - pos - 5 if pos != nil
      end
      
      # Pin these final properties onto the model.
      if resource.class == Redland::Resource
        cluster = nil
        if cluster_id != nil
          cluster = model.create_resource
          cluster.bnode("gs#{cluster_id}")
          cluster.add_property Redland::TYPE, Uri.new('http://www.pipian.com/schemas/bibo-ext/GoogleScholarCluster')
          cluster.add_property Uri.new('http://www.pipian.com/schemas/bibo-ext/clusterID'), cluster_id
          resource.add_property Uri.new('http://www.pipian.com/schemas/bibo-ext/googleScholarCluster'), cluster
        end
        if document_id != nil
          if cluster.class == Redland::Resource
            cluster.add_property Uri.new('http://www.pipian.com/schemas/bibo-ext/rootDocumentID'), document_id
          end
#          resource.add_property Uri.new('http://www.pipian.com/schemas/bibo-ext/documentID'), document_id
        end
        if issn != nil
          resource.add_property Uri.new('http://purl.org/ontology/bibo/issn'), issn
        end
        if isbn != nil
          if isbn.length == 10
            resource.add_property Uri.new('http://purl.org/ontology/bibo/isbn10'), isbn
          elsif isbn.length == 13
            resource.add_property Uri.new('http://purl.org/ontology/bibo/isbn13'), isbn
          end
        end
        if doi != nil
          resource.add_property Uri.new('http://purl.org/ontology/bibo/doi'), doi
        end
        if oclc != nil
          resource.add_property Uri.new('http://www.pipian.com/schemas/bibo-ext/oclc'), oclc
        end
      end
    end
    
    model
  end
end